from tests import get_ct_client


class TestGroupHomepages:
    @classmethod
    def setup_class(cls):
        cls.ct = get_ct_client()
        cls.homepages = cls.ct.group_homepages.list()

    def test_list(self):
        assert self.ct.group_homepages.list()

    def test_get(self):
        assert self.homepages and len(self.homepages) > 0
        assert self.ct.group_homepages.get(self.homepages[0].frontendUrl.split("/")[-1])



from datetime import date, datetime
from typing import Any, Dict, Generator, Generic, List, Optional, TypeVar, Union

from pydantic import BaseModel  # type: ignore

PydanticField = TypeVar("PydanticField")


class EmptyStrToNone(Generic[PydanticField]):
    @classmethod
    def __get_validators__(cls) -> Generator:
        yield cls.validate

    @classmethod
    def validate(cls, v: PydanticField, field: Any) -> Optional[PydanticField]:
        if v == "":
            return None
        return v


class EmptyStrToFalse(Generic[PydanticField]):
    @classmethod
    def __get_validators__(cls) -> Generator:
        yield cls.validate

    @classmethod
    def validate(cls, v: PydanticField, field: Any) -> bool:
        if v == "" or not v:
            return False
        return True


class Person(BaseModel):
    id: Optional[int] = None
    securityLevelForPerson: Optional[int] = None
    editSecurityLevelForPerson: Optional[int] = None
    title: Optional[str] = None
    firstName: Optional[str] = None
    lastName: Optional[str] = None
    nickname: Optional[str] = None
    job: Optional[str] = None
    street: Optional[str] = None
    addressAddition: Optional[str] = None
    zip: Optional[str] = None
    city: Optional[str] = None
    country: Optional[str] = None
    latitude: Optional[EmptyStrToNone[float]] = None
    longitude: Optional[EmptyStrToNone[float]] = None
    latitudeLoose: Optional[EmptyStrToNone[float]] = None
    longitudeLoose: Optional[EmptyStrToNone[float]] = None
    phonePrivate: Optional[str] = None
    phoneWork: Optional[str] = None
    mobile: Optional[str] = None
    fax: Optional[str] = None
    birthName: Optional[str] = None
    birthday: Optional[EmptyStrToNone[date]] = None
    birthplace: Optional[str] = None
    imageUrl: Optional[str] = None
    familyImageUrl: Optional[str] = None
    sexId: Optional[int] = None
    email: Optional[str] = None
    cmsUserId: Optional[str] = None
    optigemUserId: Optional[str] = None
    nationalityId: Optional[int] = None
    familyStatusId: Optional[int] = None
    weddingDate: Optional[EmptyStrToNone[date]] = None
    campusId: Optional[int] = None
    statusId: Optional[int] = None
    departmentIds: Optional[List[int]] = None
    firstContact: Optional[EmptyStrToNone[datetime]] = None
    dateOfBelonging: Optional[EmptyStrToNone[date]] = None
    dateOfEntry: Optional[EmptyStrToNone[datetime]] = None
    dateOfResign: Optional[EmptyStrToNone[datetime]] = None
    dateOfBaptism: Optional[EmptyStrToNone[date]] = None
    placeOfBaptism: Optional[str] = None
    baptisedBy: Optional[str] = None
    referredBy: Optional[str] = None
    referredTo: Optional[str] = None
    growPathId: Optional[int] = None
    isArchived: Optional[bool] = None

    # TODO: privacyPolicyAgreement
    # TODO: meta

    def __repr__(self) -> str:
        return f"<Person: {self.firstName} {self.lastName} [{self.id}]>"


class AgendaSong(BaseModel):
    songId: int
    arrangementId: int
    title: str
    arrangement: str
    category: str
    key: str
    bpm: str
    defaultArrangement: Optional[str] = None


class AgendaItem(BaseModel):
    id: int
    position: int
    type: str
    title: str
    note: Optional[str] = None
    duration: int
    start: datetime
    isBeforeEvent: bool
    song: Optional[AgendaSong] = None

    # TODO: responsible
    # TODO: serviceGroupNotes
    # TODO: meta


class Agenda(BaseModel):
    id: int
    name: str
    series: str
    isFinal: bool
    calendarId: int
    total: int
    items: List[AgendaItem]

    # TODO: meta

    def __repr__(self) -> str:
        return f"<Agenda: {self.name} [{self.id}]>"


class Service(BaseModel):
    id: int
    name: str
    serviceGroupId: int
    commentOnConfirmation: bool
    sortKey: int
    allowDecline: bool
    allowExchange: bool
    comment: str
    standard: bool
    hidePersonName: bool
    sendReminderMails: bool
    sendServiceRequestMails: Optional[bool] = None
    allowControlLiveAgenda: bool
    groupIds: Optional[str] = None
    tagIds: Optional[str] = None
    calTextTemplate: str
    allowChat: bool

    def __repr__(self) -> str:
        return f"<Service: {self.name} [{self.id}]>"


class ServiceGroup(BaseModel):
    id: int
    name: str
    sortKey: int
    viewAll: Optional[bool] = None
    campusId: Optional[int] = None
    onlyVisibleInCampusFilter: Optional[bool] = None

    def __repr__(self) -> str:
        return f"<ServiceGroup: {self.name} [{self.id}]>"


class EventService(BaseModel):
    id: int
    name: Optional[str] = None
    serviceId: int
    agreed: bool
    isValid: bool
    requestedDate: datetime
    comment: str
    counter: int
    allowChat: bool

    # TODO: person
    # TODO: requesterPerson

    def __repr__(self) -> str:
        return f"<EventService: {self.name}>"


class Event(BaseModel):
    id: int
    guid: str
    name: str
    description: Optional[str] = None
    startDate: datetime
    endDate: datetime
    chatStatus: str
    eventServices: Optional[List[EventService]] = None

    # TODO: permissions
    # TODO: calendar

    def __repr__(self) -> str:
        return (
            f"<Event: {self.startDate.day:02}."
            f"{self.startDate.month:02} {self.startDate.hour:02}:"
            f'{self.startDate.minute:02} "{self.name}" [{self.id}]>'
        )


class GroupDomainAttributes(BaseModel):
    note: str


class Group(BaseModel):
    title: str
    domainType: str
    domainIdentifier: str
    apiUrl: str
    frontendUrl: str
    imageUrl: Optional[str] = None
    domainAttributes: GroupDomainAttributes

    def __repr__(self) -> str:
        return f"<Group: {self.title}>"


class ArrangementLink(BaseModel):
    domainType: str
    domainId: str
    name: str
    filename: str
    fileUrl: str


class ArrangementFile(BaseModel):
    domainType: str
    domainId: str
    name: str
    filename: str
    fileUrl: str


class Arrangement(BaseModel):
    id: int
    name: str
    isDefault: bool
    keyOfArrangement: str
    bpm: str
    beat: str
    duration: int
    note: str
    links: List[ArrangementLink]
    files: List[ArrangementFile]

    # TODO: meta


class SongCategory(BaseModel):
    id: int
    name: str
    nameTranslated: str
    sortKey: int
    campusId: Optional[int] = None


class Song(BaseModel):
    id: int
    name: str
    category: SongCategory
    shouldPractice: bool
    author: str
    ccli: str
    copyright: str
    note: str
    arrangements: List[Arrangement]

    # TODO: meta

    def __repr__(self) -> str:
        return f"<Song: {self.name} [{self.id}]>"


class VersionInfo(BaseModel):
    build: str
    version: str

    def __repr__(self) -> str:
        return f"<Version: {self.version} (build: {self.build})>"


class PersonTag(BaseModel):
    id: int
    name: str
    count: int


class PersonDomainAttributes(BaseModel):
    firstName: str
    lastName: str
    guid: str


class PersonDomainObject(BaseModel):
    domainAttributes: PersonDomainAttributes
    imageUrl: str
    frontendUrl: str
    apiUrl: str
    domainType: str
    title: str
    domainIdentifier: str


class PersonRelationship(BaseModel):
    relative: PersonDomainObject
    degreeOfRelationship: str
    relationshipName: str
    relationshipTypeId: int

    def __repr__(self) -> str:
        return (
            f"<Relationship: {self.relative.title} " f"({self.degreeOfRelationship})>"
        )


class Setting(BaseModel):
    module: str
    attribute: str
    value: Union[dict, int, List, str]

    def __repr__(self) -> str:
        return f"<Setting: {self.module}.{self.attribute} = {self.value}>"


class CTStatus(BaseModel):
    id: int
    name: str
    shorty: str
    isMember: bool
    isSearchable: bool
    sortKey: int
    securityLevelId: int


class WikiCategory(BaseModel):
    id: int
    name: str
    sortKey: int
    campusId: Optional[int] = None
    inMenu: bool
    fileAccessWithoutPermission: bool
    nameTranslated: str

    def __repr__(self) -> str:
        return f"<WikiCategory: {self.name} [{self.id}]>"


class WikiPermission(BaseModel):
    canEdit: bool


class WikiPage(BaseModel):
    identifier: str
    wikiCategory: WikiCategory
    title: str
    version: int
    text: Optional[str] = None
    onStartpage: EmptyStrToFalse[bool]
    redirectTo: Optional[str] = None
    permissions: WikiPermission
    isMarkdown: bool

    # TODO: meta

    def __repr__(self) -> str:
        return f"<WikiPage: {self.title} [{self.identifier}]>"


class WikiSearchResult(BaseModel):
    title: str
    domainType: str
    domainIdentifier: str
    apiUrl: str
    frontendUrl: str
    imageUrl: Optional[str] = None
    preview: str

    def __repr__(self) -> str:
        return f"<WikiSearchResult: {self.title} ({self.preview})>"


class BirthdayPersonDomainAttributes(BaseModel):
    firstName: str
    lastName: str
    guid: str


class BirthdayPerson(BaseModel):
    title: str
    domainType: str
    domainIdentifier: str
    apiUrl: str
    frontendUrl: str
    imageUrl: Optional[str] = None
    domainAttributes: BirthdayPersonDomainAttributes

    def __repr__(self) -> str:
        return f"<BirthdayPerson: {self.title}>"


class Birthday(BaseModel):
    type: str
    date: date
    age: Optional[int] = None
    person: BirthdayPerson

    def __repr__(self) -> str:
        return f"<Birthday: {self.person.title} {self.date} " f"({self.age} years)>"


class ServiceRequest(BaseModel):
    id: int
    personId: int
    name: str
    serviceId: int
    agreed: bool
    isValid: bool
    requestedDate: datetime
    requesterPersonId: int
    comment: str
    counter: int

    # TODO: person
    # TODO: requesterPerson

    def __repr__(self) -> str:
        return f"<ServiceRequest: {self.name} {self.serviceId} [{self.id}]>"


class Device(BaseModel):
    id: str
    type: str
    ttl: datetime
    version: str
    createdAt: datetime
    updatedAt: datetime


class Pagination(BaseModel):
    total: int
    current: int
    limit: int
    lastPage: int


class MetaPagination(BaseModel):
    count: int
    pagination: Pagination


class Department(BaseModel):
    id: int
    name: str
    nameTranslated: Optional[str] = None
    sortKey: int

    def __repr__(self) -> str:
        return f"<Department: {self.name} [{self.id}]>"


class SearchResult(BaseModel):
    apiUrl: str
    domainIdentifier: str
    domainType: str
    frontendUrl: str
    icon: str
    imageUrl: Optional[str] = None
    title: str

    # TODO: domainAttributes

    def __repr__(self) -> str:
        return f"<SearchResult: {self.title} [{self.domainType}]>"


class Calendar(BaseModel):
    campusId: Optional[int] = None
    color: str
    eventTemplateId: Optional[int] = None
    iCalSourceUrl: Optional[str] = None
    id: int
    isPrivate: bool
    isPublic: bool
    name: str
    nameTranslated: str
    randomUrl: str
    sortKey: int

    # TODO: meta


class Address(BaseModel):
    meetingAt: Optional[str] = None
    street: Optional[str] = None
    addition: Optional[str] = None
    district: Optional[str] = None
    zip: Optional[str] = None
    city: Optional[str] = None
    country: Optional[str] = None
    latitude: Optional[EmptyStrToNone[float]] = None
    longitude: Optional[EmptyStrToNone[float]] = None


class Appointment(BaseModel):
    id: Union[int, str]
    caption: str
    note: Optional[str] = None
    address: Optional[Address] = None
    version: int
    calendar: Calendar
    information: Optional[str] = None
    image: Optional[str] = None
    link: Optional[str] = None
    isInternal: bool
    startDate: Union[date, datetime]
    endDate: Union[date, datetime]
    allDay: bool
    repeatId: int
    repeatFrequency: Optional[int] = None
    repeatUntil: Optional[str] = None
    repeatOption: Optional[int] = None

    # TODO: additions, exceptions, meta


class AgeGroup(BaseModel):
    end: Optional[int] = None
    id: int
    name: str
    nameTranslated: Optional[str] = None
    start: Optional[int] = None
    sortKey: Optional[int] = None


class GroupHomepageDomainAttributes(BaseModel):
    childGroupIds: Optional[List[int]] = None


class GroupHomepageSimple(BaseModel):
    apiUrl: Optional[str] = None
    domainAttributes: Optional[GroupHomepageDomainAttributes] = None
    frontendUrl: Optional[str] = None
    icon: Optional[str] = None
    imageUrl: Optional[str] = None
    title: Optional[str] = None

    def get_hash(self) -> str:
        if self.apiUrl:
            return self.apiUrl.split("/")[-1]

        return ""


class GroupInformation(BaseModel):
    ageGroups: Optional[List[AgeGroup]] = None
    campus: Optional[Dict] = None
    groupCategory: Optional[str] = None
    groupPlaces: Optional[List[Dict]] = None
    imageUrl: Optional[str] = None
    leader: Optional[List[PersonDomainObject]] = None
    meetingTime: Optional[str] = None
    note: Optional[str] = None
    targetGroup: Optional[Dict] = None
    weekday: Optional[Dict] = None


class GroupDetail(BaseModel):
    allowWaitinglist: Optional[bool] = None
    autoAccept: Optional[bool] = None
    canSignUp: Optional[bool] = None
    children: Optional[List[int]] = None
    currentMemberCount: Optional[int] = None
    id: Optional[int] = None
    information: Optional[GroupInformation] = None
    maxMemberCount: Optional[int] = None
    name: Optional[str] = None
    requestedPlacesCount: Optional[int] = None
    requestedWaitinglistPlacesCount: Optional[int] = None
    signUpConditions: Optional[Dict] = None
    signUpHeadline: Optional[str] = None
    signUpPersons: Optional[Dict] = None


class GroupHomepage(BaseModel):
    defaultView: Optional[str] = None
    filter: Optional[List[Dict]] = None
    groups: List[GroupDetail]
    id: Optional[int] = None
    isEnabled: Optional[bool] = None
    meta: Optional[Dict] = None
    orderDirection: Optional[str] = None
    parentGroup: Optional[int] = None
    randomUrl: Optional[str] = None
    showFilter: Optional[bool] = None
    showGroups: Optional[bool] = None
    showLeader: Optional[bool] = None
    showMap: Optional[bool] = None
    sortBy: Optional[str] = None



"""
Wiki
====

Endpoints for the wiki.

"""

from typing import Any, Dict, List, Optional

from churchtools.ct_types import WikiCategory, WikiPage, WikiSearchResult


class Wiki:
    def __init__(self, ct: Any) -> None:
        """Initialize a Status object."""

        self.__ct = ct

    def categories(self) -> List[WikiCategory]:
        """Get all wiki categories.

        :returns: List of wiki categories
        :rtype: List[WikiCategory]
        """

        res = self.__ct.make_request("wiki/categories")

        ctgs = []
        if res and "data" in res:
            for item in res["data"]:
                ctg = WikiCategory(**item)
                ctgs.append(ctg)

        return ctgs

    def pages(self, category_id: int) -> List[WikiPage]:
        """Get all pages in a wiki category.

        :param category_id: The ID of the category
        :type category_id: int
        :returns: All pages in the category
        :rtype: List[WikiPage]
        """

        res = self.__ct.make_request(f"wiki/categories/{category_id}/pages")

        pgs = []
        if res and "data" in res:
            for item in res["data"]:
                pg = WikiPage(**item)
                pgs.append(pg)

        return pgs

    def page(
        self, category_id: int, identifier: str, version: Optional[int] = None
    ) -> Optional[WikiPage]:
        """Get a wiki page.

        :param category_id: The ID of the category
        :type category_id: int
        :param identifier: The ID of the page
        :type identifier: int
        :param version: The revision of the page
        :type version: int
        :returns: The wiki page
        :rtype: WikiPage
        """

        if version:
            res = self.__ct.make_request(
                f"wiki/categories/{category_id}/"
                f"pages/{identifier}/"
                f"versions/{version}"
            )
        else:
            res = self.__ct.make_request(
                f"wiki/categories/{category_id}/" f"pages/{identifier}"
            )

        if res and "data" in res:
            return WikiPage(**res["data"])

        return None

    def versions(self, category_id: int, identifier: str) -> List[WikiPage]:
        """Get the versions of a wiki page.

        :param category_id: The ID of the category
        :type category_id: int
        :param identifier: The ID of the page
        :type identifier: int
        :returns: The wiki page
        :rtype: WikiPage
        """

        res = self.__ct.make_request(
            f"wiki/categories/{category_id}/" f"pages/{identifier}/versions"
        )

        pgs = []
        if res and "data" in res:
            for item in res["data"]:
                pgs.append(WikiPage(**item))

        return pgs

    def search(
        self, query: str, wiki_category_ids: Optional[List[int]] = None
    ) -> List[WikiSearchResult]:
        """Search for a query.

        :param query: The search query
        :type query: str
        :param wiki_category_ids: Search only in these categories. Search in
            all categories, if this is None
        :returns: The search results
        :rtype: List[WikiSearchResult]
        """

        params: Dict[str, Any] = {}

        params["query"] = query
        if wiki_category_ids:
            params["wiki_category_ids"] = wiki_category_ids

        res = self.__ct.make_request("wiki/search", params)

        results = []
        if res and "data" in res:
            for item in res["data"]:
                result = WikiSearchResult(**item)
                results.append(result)

        return results



"""
Events
======

Endpoints for the event module.

TODO:
    - export
    - ical
    - send
    - chat

"""

from datetime import datetime
from typing import Any, List, Optional

from churchtools.ct_types import Agenda, Event, Song


class Events:
    def __init__(self, ct: Any) -> None:
        """Initialize an Events object."""

        self.__ct = ct

    def get(self, event_id: int) -> Event | None:
        """Get a single event by id.

        :param event_id: The event id
        :type event_id: int
        :returns: The event
        :rtype: Event
        """

        route = f"events/{event_id}"
        res = self.__ct.make_request(route)

        if res and res["data"]:
            return Event(**res["data"])

        return None

    def list(
        self,
        from_date: Optional[datetime] = None,
        to_date: Optional[datetime] = None,
        include: Optional[str] = None,
    ) -> List[Event]:
        """List upcoming events, or events from or to a date.

        :param from_date: List events after this date. Default: today
        :type from_date: datetime
        :param to_date: List events up to this date. Default: two months from now
        :type to_date: datetime
        :param include: Include additional data (e.g. "eventServices")
        :type include: str
        :returns: A list of events
        :rtype: List[Event]
        """

        params = {}

        if to_date:
            params["to_date"] = (
                f"{to_date.year}-{to_date.month:02d}-" f"{to_date.day:02d}"
            )
        if from_date:
            params["from_date"] = (
                f"{from_date.year}-{from_date.month:02d}-" f"{from_date.day:02d}"
            )
        if include:
            params["include"] = include

        res = self.__ct.make_request("events", params)

        events = []
        if "data" in res:
            for item in res["data"]:
                ev = Event(**item)
                events.append(ev)

        return events

    def agenda(self, event_id: int) -> Agenda | None:
        """Get the agenda of an event.

        :param event_id: The ID of the event
        :type event_id: int
        :returns: The agenda of the event
        :rtype: Agenda
        """

        route = f"events/{event_id}/agenda"
        res = self.__ct.make_request(route)

        if res and res["data"]:
            return Agenda(**res["data"])

        return None

    def songs(self, event_id: int) -> List[Song]:
        """Get the songs of an event.

        :param event_id: The ID of the event
        :type event_id: int
        :returns: A list of songs, that is set for the event (unordered).
        :rtype: List[Song]
        """

        route = f"events/{event_id}/agenda/songs"
        res = self.__ct.make_request(route)

        sngs = []
        for item in res["data"]:
            sng = Song(**item)
            sngs.append(sng)

        return sngs

    def masterdata(self) -> dict:
        """Get masterdata for module `event`.

        The master data are the backbone of ChurchTools. This endpoint returns
        all data for that module to work with. Different endpoints don't
        include the master data directly but only state the ID for this data
        and this endpoint provides the data with all its details.
        """

        route = "event/masterdata"
        res = self.__ct.make_request(route)

        return res



from tests import get_ct_client


class TestEvents:
    @classmethod
    def setup_class(cls):
        cls.ct = get_ct_client()
        cls.events = cls.ct.events.list()

    def test_list(self):
        assert self.ct.events.list(include="eventServices")

    def test_get(self):
        assert self.events and len(self.events) > 0
        assert self.ct.events.get(self.events[0].id)

    def test_agenda(self):
        # TODO: find an event with agenda more reliably
        next_services = [evt for evt in self.events if "Gottesdienst" in evt.name]
        assert len(next_services) > 0
        assert self.ct.events.agenda(next_services[0].id)

    def test_songs(self):
        # TODO: find an event with songs more reliably
        next_services = [evt for evt in self.events if "Gottesdienst" in evt.name]
        assert len(next_services) > 0
        assert self.ct.events.songs(next_services[0].id)

    def test_masterdata(self):
        assert self.ct.events.masterdata()



from tests import get_ct_client


class TestPersons:
    @classmethod
    def setup_class(cls):
        cls.ct = get_ct_client()
        cls.me = cls.ct.general.whoami()

    def test_get(self):
        assert self.ct.persons.get(self.me.id)

    def test_list(self):
        assert self.ct.persons.list()

    # def test_tags(self):
    #    assert self.ct.persons.tags(self.me.id)

    def test_relationships(self):
        assert self.ct.persons.relationships(self.me.id)

    def test_events(self):
        assert self.ct.persons.events(self.me.id)

    def test_groups(self):
        assert self.ct.persons.groups(self.me.id)

    def test_settings(self):
        settings = self.ct.persons.settings(self.me.id)
        assert settings
        assert self.ct.persons.setting(
            self.me.id, settings[0].module, settings[0].attribute
        )

    def test_birthdays(self):
        assert self.ct.persons.birthdays()

    def test_service_requests(self):
        servicerequests = self.ct.persons.servicerequests(self.me.id)
        assert servicerequests
        assert self.ct.persons.servicerequest(self.me.id, servicerequests[0].id)

    def test_devices(self):
        devices = self.ct.persons.devices(self.me.id)
        assert devices
        assert self.ct.persons.device(self.me.id, devices[0].id)

    def test_logintoken(self):
        assert self.ct.persons.logintoken(self.me.id)

    def test_masterdata(self):
        assert self.ct.persons.masterdata()



from tests import get_ct_client


class TestWiki:
    @classmethod
    def setup_class(cls):
        cls.ct = get_ct_client()

    def test_wiki(self):
        categories = self.ct.wiki.categories()
        assert categories
        pages = self.ct.wiki.pages(categories[0].id)
        assert pages
        assert self.ct.wiki.page(
            categories[0].id, pages[0].identifier, pages[0].version
        )
        assert self.ct.wiki.versions(categories[0].id, pages[0].identifier)

    def test_search(self):
        assert self.ct.wiki.search("Gott")



from tests import get_ct_client


class TestServices:
    @classmethod
    def setup_class(cls):
        cls.ct = get_ct_client()

    def test_list(self):
        self.ct.services.list()

    def test_groups(self):
        self.ct.services.groups()



from tests import get_ct_client


class TestSongs:
    @classmethod
    def setup_class(cls):
        cls.ct = get_ct_client()
        cls.songs, _ = cls.ct.songs.list()

    def test_list(self):
        assert self.ct.songs.list()

    def test_get(self):
        assert self.songs
        assert self.ct.songs.get(self.songs[0].id)



from tests import get_ct_client


class TestDepartments:
    @classmethod
    def setup_class(cls):
        cls.ct = get_ct_client()

    def test_list(self):
        assert self.ct.departments.list()



"""
Status
======

CRUD methods for status field

These are the statuses that a person can have.
(e.g. "Visitor", "Friend", "Member", etc.)

TODO:

    - new status
    - status DELETE, PUT

"""

from typing import Any, List, Optional

from churchtools.ct_types import CTStatus


class Status:
    def __init__(self, ct: Any) -> None:
        """Initialize a Status object."""

        self.__ct = ct

    def get_all(self) -> List[CTStatus]:
        """Get all statuses from the API.

        :returns: A list of all status
        :rtype: List[CTStatus]
        """

        res = self.__ct.make_request("statuses")

        statuses = []
        if res and "data" in res:
            for item in res["data"]:
                statuses.append(CTStatus(**item))

        return statuses

    def get(self, status_id: int) -> Optional[CTStatus]:
        """Get a status by ID.

        :param status_id: The ID of the status
        :type status_id: int
        :returns: The status
        :rtype: CTStatus
        """

        route = f"statuses/{status_id}"
        res = self.__ct.make_request(route)

        if res and "data" in res:
            return CTStatus(**res["data"])

        return None



from tests import get_ct_client


class TestCalendar:
    @classmethod
    def setup_class(cls):
        cls.ct = get_ct_client()

    def test_list_and_appointments(self):
        calendars = self.ct.calendars.list()
        public_calendars = [calendar for calendar in calendars if calendar.isPublic]
        assert len(public_calendars) > 0
        print(public_calendars[0])
        assert self.ct.calendars.appointments([public_calendars[0].id])



"""
Department
==========

Find out about departments in ChurchTools

"""

from typing import Any, List

from churchtools.ct_types import Department


class Departments:
    def __init__(self, ct: Any) -> None:
        """Initialize a Persons object."""

        self.__ct = ct

    def list(self) -> List[Department]:
        """Returns all departments"""

        res = self.__ct.make_request("departments")

        departments = []
        if res and "data" in res:
            for item in res["data"]:
                department = Department(**item)
                departments.append(department)

        return departments



"""
Person
======

Find out about persons in ChurchTools

TODO:

    - information
    - persons POST
    - duplicates
    - properties
    - person DELETE, PATCH
    - archive
    - device DELETE, PUT
    - invite
    - merge GET, PATCH
    - servicerequest DELETE, PUT
    - undo servicerequest
    - attribute DELETE, PUT

"""

from datetime import date, datetime
from typing import Any, Dict, List, Optional, Tuple

from churchtools.ct_types import (
    Birthday,
    Device,
    Event,
    Group,
    MetaPagination,
    Person,
    PersonRelationship,
    PersonTag,
    ServiceRequest,
    Setting,
)


class Persons:
    def __init__(self, ct: Any) -> None:
        """Initialize a Persons object."""

        self.__ct = ct

    def get(self, person_id: int) -> Optional[Person]:
        """Get a person by id.

        :param person_id: The ID of the person
        :type person_id: int
        :returns: A person object
        :rtype: Person
        """

        route = f"persons/{person_id}"
        res = self.__ct.make_request(route)

        if res and "data" in res:
            return Person(**res["data"])

        return None

    def list(
        self,
        ids: Optional[List[int]] = None,
        status_ids: Optional[List[int]] = None,
        campus_ids: Optional[List[int]] = None,
        birthday_before: Optional[datetime] = None,
        birthday_after: Optional[datetime] = None,
        is_archived: bool = False,
        page: int = 1,
        limit: int = 10,
    ) -> Tuple[List[Person], Optional[MetaPagination]]:
        """Returns a list of persons, that the logged in user can see.

        :param ids: A list of IDs that is to be queried
        :type ids: List[int]
        :param status_ids: A list of status IDs that is to be queried
        :type status_ids: List[int]
        :param campus_ids: A list of campus IDs that is to be queried
        :type campus_ids: List[int]
        :param birthday_before: Only list persons born before this date
        :type birthday_before: datetime
        :param birthday_after: Only list persons born after this date
        :type birthday_after: datetime
        :param is_archived: List archived persons
        :type is_archived: bool
        :param page: Get this result page number
        :type page: int
        :param limit: Return this many results
        :type limit: int
        :returns: A list of persons that match the query
        :rtype: List[Person]
        """

        params: Dict[str, Any] = {}

        if ids:
            params["ids"] = ids
        if status_ids:
            params["status_ids"] = status_ids
        if campus_ids:
            params["campus_ids"] = campus_ids
        if birthday_before:
            params["birthday_before"] = (
                f"{birthday_before.year}-{birthday_before.month:02d}-"
                f"{birthday_before.day:02d}"
            )
        if birthday_after:
            params["birthday_after"] = (
                f"{birthday_after.year}-{birthday_after.month:02d}-"
                f"{birthday_after.day:02d}"
            )
        if is_archived:
            params["is_archived"] = is_archived
        if page:
            params["page"] = page
        if limit:
            params["limit"] = limit

        res = self.__ct.make_request("persons", params=params)

        persons = []
        if res and "data" in res:
            for item in res["data"]:
                pers = Person(**item)
                persons.append(pers)

        pagination = None
        if res and "meta" in res:
            pagination = MetaPagination(**res["meta"])

        return persons, pagination

    def tags(self, person_id: int) -> List[PersonTag]:
        """Get the tags of a person.

        :param person_id: The ID of the person
        :type person_id: int
        :returns: A list of tags for the person
        :rtype: List[PersonTag]
        """

        route = f"persons/{person_id}/tags"
        res = self.__ct.make_request(route)

        tags = []
        if res and "data" in res:
            for item in res["data"]:
                tags.append(PersonTag(**item))

        return tags

    def relationships(self, person_id: int) -> List[PersonRelationship]:
        """Get relationships for a person.

        :param person_id: The ID of the person
        :type person_id: int
        :returns: A list of relationships
        :rtype: List[PersonRelationship]
        """

        route = f"persons/{person_id}/relationships"
        res = self.__ct.make_request(route)

        rls = []
        if res and "data" in res:
            for item in res["data"]:
                rl = PersonRelationship(**item)
                rls.append(rl)

        return rls

    def events(
        self, person_id: int, from_date: Optional[datetime] = None
    ) -> List[Event]:
        """Get events that a person is involved in.

        :param person_id: The ID of the person
        :type person_id: int
        :param from_date: Only query events after this date
        :type from_date: datetime
        :returns: A list of events
        :rtype: List[Event]
        """

        params = {}
        if from_date:
            params["from"] = (
                f"{from_date.year}-{from_date.month:02d}-" f"{from_date.day:02d}"
            )

        route = f"persons/{person_id}/events"
        res = self.__ct.make_request(route, params)

        evs = []
        if "data" in res:
            for item in res["data"]:
                ev = Event(**item)
                evs.append(ev)

        return evs

    def groups(
        self,
        person_id: int,
        show_overdue_groups: bool = False,
        show_inactive_groups: bool = False,
    ) -> List[Group]:
        """Get events that a person is part of.

        :param person_id: The ID of the person
        :type person_id: int
        :param show_overdue_groups: Show overdue groups
        :type show_overdue_groups: bool
        :param show_inactive_groups: Show inactive groups
        :type show_inactive_groups: bool
        :returns: A list of groups
        :rtype: List[Group]
        """

        params = {}
        if show_overdue_groups:
            params["show_overdue_groups"] = show_overdue_groups
        if show_inactive_groups:
            params["show_inactive_groups"] = show_inactive_groups

        route = f"persons/{person_id}/groups"
        res = self.__ct.make_request(route, params)

        grps = []
        for item in res["data"]:
            grp = Group(**item["group"])
            grps.append(grp)

        return grps

    def settings(self, person_id: int, module: Optional[str] = None) -> List[Setting]:
        """Get the settings for a person.

        :param person_id: The ID of the person
        :type person_id: int
        :param module: Only list settings for a module
        :type module: str
        :returns: A list of settings
        :rtype: List[Setting]
        """

        route = f"persons/{person_id}/settings"
        if module:
            route = f"{route}/{module}"

        res = self.__ct.make_request(route)

        sttngs = []

        for item in res["data"]:
            sttng = Setting(**item)
            sttngs.append(sttng)

        return sttngs

    def setting(self, person_id: int, module: str, attribute: str) -> Setting:
        """Get a setting for a person.

        :param person_id: The ID of the person
        :type person_id: int
        :param module: The settings module
        :type module: str
        :param attribute: The wanted attribute
        :type attribute: str
        :returns: The setting
        :rtype: Setting
        """

        route = f"persons/{person_id}/settings/{module}/{attribute}"

        res = self.__ct.make_request(route)

        sttng = Setting(**res["data"])
        return sttng

    def birthdays(
        self, start_date: Optional[date] = None, end_date: Optional[date] = None
    ) -> List[Birthday]:
        """Get all birthdays in a time span.

        :param start_date: Start date of the timespan (default: yesterday)
        :type start_date: date
        :param end_date: End date of the timespan (default: 30 days from now)
        :type end_date: date
        :returns: A list of birthdays, including ages and persons
        :rtype: List[Birthday]
        """

        route = "persons/birthdays"

        params = {}
        if start_date:
            params["start_date"] = start_date.strftime("%Y-%m-%d")
        if end_date:
            params["end_date"] = end_date.strftime("%Y-%m-%d")

        res = self.__ct.make_request(route, params)

        bdays = []

        for item in res["data"]:
            bday = Birthday(**item)
            bdays.append(bday)

        return bdays

    def servicerequests(self, person_id: int) -> List[ServiceRequest]:
        """Get all service requests for a person.

        :param person_id: The ID of the person
        :type person_id: int
        :returns: A list of service requests
        :rtype: List[ServiceRequest]
        """

        route = f"persons/{person_id}/servicerequests"
        res = self.__ct.make_request(route)

        rqsts = []

        for item in res["data"]:
            rqst = ServiceRequest(**item)
            rqsts.append(rqst)

        return rqsts

    def servicerequest(self, person_id: int, servicerequest_id: int) -> ServiceRequest:
        """Get all service requests for a person.

        :param person_id: The ID of the person
        :type person_id: int
        :param servicerequest_id: The ID of the service request
        :type servicerequest_id: int
        :returns: A service request
        :rtype: ServiceRequest
        """

        route = f"persons/{person_id}/servicerequests/{servicerequest_id}"
        res = self.__ct.make_request(route)

        rqst = ServiceRequest(**res["data"])
        return rqst

    def devices(self, person_id: int) -> List[Device]:
        """Get all in ChurchTools registered devices of a person.

        These are all registered smartphones/tablets with the CT app.

        :param person_id: The ID of the person
        :type person_id: int
        :returns: A list of registered devices
        :rtype: List[Device]
        """

        route = f"persons/{person_id}/devices"
        res = self.__ct.make_request(route)

        dvcs = []
        for item in res["data"]:
            dvc = Device(**item)
            dvcs.append(dvc)

        return dvcs

    def device(self, person_id: int, device_id: str) -> Device:
        """Get all information about one device.

        :param person_id: The ID of the person
        :type person_id: int
        :param device_id: The ID of the device
        :type device_id: str
        :returns: A list of registered devices
        :rtype: List[Device]
        """

        route = f"persons/{person_id}/devices/{device_id}"
        res = self.__ct.make_request(route)

        dvc = Device(**res["data"])
        return dvc

    def logintoken(self, person_id: int) -> str:
        """Fetch login token for person.

        If a token does not yet exist, a new one is created on the fly.
        """

        route = f"persons/{person_id}/logintoken"
        res = self.__ct.make_request(route)

        return res["data"]

    def masterdata(self) -> dict:
        """Fetch all master data for the module `people & groups`.

        The master data are the backbone of ChurchTools. You can add new db
        fields, or change the available countries. This endpoint returns all
        data for that module to work with. Different endpoints don't include
        the master data directly but only state the ID for this data and this
        endpoint provides the data with all its details.
        """

        route = "person/masterdata"
        res = self.__ct.make_request(route)

        return res



from tests import get_ct_client


class TestGroups:
    @classmethod
    def setup_class(cls):
        cls.ct = get_ct_client()

    def test_agegroups(self):
        assert self.ct.groups.agegroups()



import os

from setuptools import setup


# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name="churchtools",
    version="0.2.1",
    author="Philipp Glaum",
    author_email="p@pglaum.de",
    description=("A library for the ChurchTools API"),
    license="GPLv3",
    keywords="api churchtools",
    url="https://github.com/pglaum/pychurchtools",
    packages=["churchtools"],
    install_requires=[
        "pydantic",
        "requests",
    ],
    extras_require={
        "tests": [
            "black",
            "mypy",
            "pytest",
            "types-requests",
        ],
    },
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
    ],
)



from tests import get_ct_client


class TestGeneral:
    @classmethod
    def setup_class(cls):
        cls.ct = get_ct_client()

    def test_info(self):
        assert self.ct.general.info()

    def test_whoami(self):
        assert self.ct.general.whoami()

    def test_search(self):
        assert self.ct.general.search("Gott")



from tests import get_ct_client


class TestStatus:
    @classmethod
    def setup_class(cls):
        cls.ct = get_ct_client()

    def test_get(self):
        statuses = self.ct.status.get_all()
        assert statuses
        assert self.ct.status.get(statuses[0].id)
